
from urllib3.connection import HTTPConnection, HTTPSConnection
from urllib3.connectionpool import HTTPConnectionPool, HTTPSConnectionPool


def _output(self, s):
    """Host header should always be first"""

    if s.lower().startswith(b'host: '):
        self._buffer.insert(1, s)
    else:
        self._buffer.append(s)


HTTPConnectionPool.ConnectionCls = type(
    'RevProxyHTTPConnection',
    (HTTPConnection,),
    {'_output': _output},
)

HTTPSConnectionPool.ConnectionCls = type(
    'RevProxyHTTPSConnection',
    (HTTPSConnection,),
    {'_output': _output}
)




import re

import logging

from wsgiref.util import is_hop_by_hop

try:
    from http.cookies import SimpleCookie
    COOKIE_PREFIX = ''
except ImportError:
    from Cookie import SimpleCookie
    COOKIE_PREFIX = 'Set-Cookie: '


#: List containing string constant that are used to represent headers that can
#: be ignored in the required_header function
IGNORE_HEADERS = (
    'HTTP_ACCEPT_ENCODING',  # We want content to be uncompressed so
                             # we remove the Accept-Encoding from
                             # original request
    'HTTP_HOST',
    'HTTP_REMOTE_USER',
)


# Default from HTTP RFC 2616
#   See: http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.7.1
#: Variable that represent the default charset used
DEFAULT_CHARSET = 'latin-1'

#: List containing string constants that represents possible html content type
HTML_CONTENT_TYPES = (
    'text/html',
    'application/xhtml+xml'
)

#: Variable used to represent a minimal content size required for response
#: to be turned into stream
MIN_STREAMING_LENGTH = 4 * 1024  # 4KB

#: Regex used to find charset in a html content type
_get_charset_re = re.compile(r';\s*charset=(?P<charset>[^\s;]+)', re.I)


def is_html_content_type(content_type):
    """Function used to verify if the parameter is a proper html content type

    :param content_type: String variable that represent a content-type
    :returns:  A boolean value stating if the content_type is a valid html
               content type
    """
    for html_content_type in HTML_CONTENT_TYPES:
        if content_type.startswith(html_content_type):
            return True

    return False


def should_stream(proxy_response):
    """Function to verify if the proxy_response must be converted into
    a stream.This will be done by checking the proxy_response content-length
    and verify if its length is bigger than one stipulated
    by MIN_STREAMING_LENGTH.

    :param proxy_response: An Instance of urllib3.response.HTTPResponse
    :returns: A boolean stating if the proxy_response should
              be treated as a stream
    """
    content_type = proxy_response.headers.get('Content-Type')

    if is_html_content_type(content_type):
        return False

    try:
        content_length = int(proxy_response.headers.get('Content-Length', 0))
    except ValueError:
        content_length = 0

    if not content_length or content_length > MIN_STREAMING_LENGTH:
        return True

    return False


def get_charset(content_type):
    """Function used to retrieve the charset from a content-type.If there is no
    charset in the content type then the charset defined on DEFAULT_CHARSET
    will be returned

    :param  content_type:   A string containing a Content-Type header
    :returns:               A string containing the charset
    """
    if not content_type:
        return DEFAULT_CHARSET

    matched = _get_charset_re.search(content_type)
    if matched:
        # Extract the charset and strip its double quotes
        return matched.group('charset').replace('"', '')
    return DEFAULT_CHARSET


def required_header(header):
    """Function that verify if the header parameter is a essential header

    :param header:  A string represented a header
    :returns:       A boolean value that represent if the header is required
    """
    if header in IGNORE_HEADERS:
        return False

    if header.startswith('HTTP_') or header == 'CONTENT_TYPE':
        return True

    return False


def set_response_headers(response, response_headers):
    # check for Django 3.2 headers interface
    # https://code.djangoproject.com/ticket/31789
    # check and set pointer before loop to improve efficiency
    if hasattr(response, 'headers'):
        headers = response.headers
    else:
        headers = response

    for header, value in response_headers.items():
        if is_hop_by_hop(header) or header.lower() == 'set-cookie':
            continue

        headers[header] = value

    if hasattr(response, 'headers'):
        logger.debug('Response headers: %s', response.headers)
    else:
        logger.debug('Response headers: %s', getattr(response, '_headers', None))


def normalize_request_headers(request):
    r"""Function used to transform header, replacing 'HTTP\_' to ''
    and replace '_' to '-'

    :param request:  A HttpRequest that will be transformed
    :returns:        A dictionary with the normalized headers
    """
    norm_headers = {}
    for header, value in request.META.items():
        if required_header(header):
            norm_header = header.replace('HTTP_', '').title().replace('_', '-')
            norm_headers[norm_header] = value

    return norm_headers


def encode_items(items):
    """Function that encode all elements in the list of items passed as
    a parameter

    :param items:  A list of tuple
    :returns:      A list of tuple with all items encoded in 'utf-8'
    """
    encoded = []
    for key, values in items:
        for value in values:
            encoded.append((key.encode('utf-8'), value.encode('utf-8')))
    return encoded


logger = logging.getLogger('revproxy.cookies')


def cookie_from_string(cookie_string, strict_cookies=False):
    """Parser for HTTP header set-cookie
    The return from this function will be used as parameters for
    django's response.set_cookie method. Because set_cookie doesn't
    have parameter comment, this cookie attribute will be ignored.

    :param  cookie_string: A string representing a valid cookie
    :param  strict_cookies: Whether to only accept RFC-compliant cookies
    :returns: A dictionary containing the cookie_string attributes
    """

    if strict_cookies:

        cookies = SimpleCookie(COOKIE_PREFIX + cookie_string)
        if not cookies.keys():
            return None
        cookie_name, = cookies.keys()
        cookie_dict = {k: v for k, v in cookies[cookie_name].items()
                       if v and k != 'comment'}
        cookie_dict['key'] = cookie_name
        cookie_dict['value'] = cookies[cookie_name].value
        return cookie_dict

    else:
        valid_attrs = ('path', 'domain', 'comment', 'expires',
                       'max-age', 'httponly', 'secure', 'samesite')

        cookie_dict = {}

        cookie_parts = cookie_string.split(';')
        try:
            key, value = cookie_parts[0].split('=', 1)
            cookie_dict['key'], cookie_dict['value'] = key, unquote(value)
        except ValueError:
            logger.warning('Invalid cookie: `%s`', cookie_string)
            return None

        if cookie_dict['value'].startswith('='):
            logger.warning('Invalid cookie: `%s`', cookie_string)
            return None

        for part in cookie_parts[1:]:
            if '=' in part:
                attr, value = part.split('=', 1)
                value = value.strip()
            else:
                attr = part
                value = ''

            attr = attr.strip().lower()
            if not attr:
                continue

            if attr in valid_attrs:
                if attr in ('httponly', 'secure'):
                    cookie_dict[attr] = True
                elif attr in 'comment':
                    # ignoring comment attr as explained in the
                    # function docstring
                    continue
                elif attr == 'max-age':
                    # The cookie uses 'max-age' but django's
                    # set_cookie uses 'max_age'
                    cookie_dict['max_age'] = unquote(value)
                else:
                    cookie_dict[attr] = unquote(value)
            else:
                logger.warning('Unknown cookie attribute %s', attr)

        return cookie_dict


def unquote(value):
    """Remove wrapping quotes from a string.

    :param value: A string that might be wrapped in double quotes, such
                  as a HTTP cookie value.
    :returns: Beginning and ending quotes removed and escaped quotes (``\"``)
              unescaped
    """
    if len(value) > 1 and value[0] == '"' and value[-1] == '"':
        value = value[1:-1].replace(r'\"', '"')
    return value




import re

from io import StringIO

import logging

try:
    from django.utils.six import string_types
except ImportError:
    # Django 3 has no six
    string_types = str
from django.template import loader

try:
    from diazo.compiler import compile_theme
except ImportError:
    #: Variable used to identify if diazo is available
    HAS_DIAZO = False
else:
    HAS_DIAZO = True
    from lxml import etree

from .utils import get_charset, is_html_content_type

#: Regex used to find the doctype-header in a html content
doctype_re = re.compile(br"^<!DOCTYPE\s[^>]+>\s*", re.MULTILINE)
#: String used to verify if request has a 'HTTP_X_DIAZO_OFF' header
DIAZO_OFF_REQUEST_HEADER = 'HTTP_X_DIAZO_OFF'
#: String used to verify if request has a 'X-Diazo-Off' header
DIAZO_OFF_RESPONSE_HEADER = 'X-Diazo-Off'


def asbool(value):
    """Function used to convert certain string values into an appropriated
    boolean value.If value is not a string the built-in python
    bool function will be used to convert the passed parameter

    :param      value: an object to be converted to a boolean value
    :returns:   A boolean value
    """

    is_string = isinstance(value, string_types)

    if is_string:
        value = value.strip().lower()
        if value in ('true', 'yes', 'on', 'y', 't', '1',):
            return True
        elif value in ('false', 'no', 'off', 'n', 'f', '0'):
            return False
        else:
            raise ValueError("String is not true/false: %r" % value)
    else:
        return bool(value)


class DiazoTransformer(object):
    """Class used to make a diazo transformation on a request or a response"""

    def __init__(self, request, response):
        self.request = request
        self.response = response
        self.log = logging.getLogger('revproxy.transformer')
        self.log.info("DiazoTransformer created")

    def should_transform(self):
        """Determine if we should transform the response

        :returns:  A boolean value
        """

        if not HAS_DIAZO:
            self.log.info("HAS_DIAZO: false")
            return False

        if asbool(self.request.META.get(DIAZO_OFF_REQUEST_HEADER)):
            self.log.info("DIAZO_OFF_REQUEST_HEADER in request.META: off")
            return False

        if asbool(self.response.get(DIAZO_OFF_RESPONSE_HEADER)):
            self.log.info("DIAZO_OFF_RESPONSE_HEADER in response.get: off")
            return False

        if self.request.is_ajax():
            self.log.info("Request is AJAX")
            return False

        if self.response.streaming:
            self.log.info("Response has streaming")
            return False

        content_type = self.response.get('Content-Type')
        if not is_html_content_type(content_type):
            self.log.info("Content-type: false")
            return False

        content_encoding = self.response.get('Content-Encoding')
        if content_encoding in ('zip', 'compress'):
            self.log.info("Content encode is %s", content_encoding)
            return False

        status_code = str(self.response.status_code)
        if status_code.startswith('3') or \
                status_code == '204' or \
                status_code == '401':
            self.log.info("Status code: %s", status_code)
            return False

        if len(self.response.content) == 0:
            self.log.info("Response Content is EMPTY")
            return False

        self.log.info("Transform")
        return True

    def transform(self, rules, theme_template, is_html5, context_data=None):
        """Method used to make a transformation on the content of
        the http response based on the rules and theme_templates
        passed as paremters

        :param  rules: A file with a set of diazo rules to make a
                       transformation over the original response content
        :param  theme_template: A file containing the template used to format
                                the the original response content
        :param    is_html5: A boolean parameter to identify a html5 doctype
        :returns: A response with a content transformed based on the rules and
                  theme_template
        """

        if not self.should_transform():
            self.log.info("Don't need to be transformed")
            return self.response

        theme = loader.render_to_string(theme_template, context=context_data,
                                        request=self.request)
        output_xslt = compile_theme(
            rules=rules,
            theme=StringIO(theme),
        )

        transform = etree.XSLT(output_xslt)
        self.log.debug("Transform: %s", transform)

        charset = get_charset(self.response.get('Content-Type'))

        try:
            decoded_response = self.response.content.decode(charset)
        except UnicodeDecodeError:
            decoded_response = self.response.content.decode(charset, 'ignore')
            self.log.warning("Charset is {} and type of encode used in file is\
                              different. Some unknown characteres might be\
                              ignored.".format(charset))

        content_doc = etree.fromstring(decoded_response,
                                       parser=etree.HTMLParser())

        self.response.content = transform(content_doc)

        if is_html5:
            self.set_html5_doctype()

        self.reset_headers()

        self.log.debug("Response transformer: %s", self.response)
        return self.response

    def reset_headers(self):
        """This method remove the header Content-Length entry
        from the response
        """
        self.log.info("Reset header")
        del self.response['Content-Length']

    def set_html5_doctype(self):
        """Method used to transform a doctype in to a properly html5 doctype
        """
        doctype = b'<!DOCTYPE html>\n'
        content = doctype_re.subn(doctype, self.response.content, 1)[0]
        self.response.content = content



# -*- coding: utf-8 -*-

import os
import re
import sys
import mimetypes
import logging

import urllib3

try:
    from django.utils.six.moves.urllib.parse import (
        urlparse, urlencode, quote_plus)
except ImportError:
    # Django 3 has no six
    from urllib.parse import urlparse, urlencode, quote_plus

from django.shortcuts import redirect
from django.views.generic import View
from django.utils.decorators import classonlymethod
from django.views.generic.base import ContextMixin

from .exceptions import InvalidUpstream
from .response import get_django_response
from .transformer import DiazoTransformer
from .utils import normalize_request_headers, encode_items

# Chars that don't need to be quoted. We use same than nginx:
#   https://github.com/nginx/nginx/blob/nginx-1.9/src/core/ngx_string.c
#   (Lines 1433-1449)
QUOTE_SAFE = r'<.;>\(}*+|~=-$/_:^@)[{]&\'!,"`'


ERRORS_MESSAGES = {
    'upstream-no-scheme': ("Upstream URL scheme must be either "
                           "'http' or 'https' (%s).")
}

HTTP_POOLS = urllib3.PoolManager()


class ProxyView(View):
    """View responsable by excute proxy requests, process and return
    their responses.

    """
    _upstream = None

    add_x_forwarded = False
    add_remote_user = False
    default_content_type = 'application/octet-stream'
    retries = None
    rewrite = tuple()  # It will be overrided by a tuple inside tuple.
    strict_cookies = False
    #: Do not send any body if it is empty (put ``None`` into the ``urlopen()``
    #: call).  This is required when proxying to Shiny apps, for example.
    suppress_empty_body = False

    # The buffering amount for streaming HTTP response(in bytes), response will
    # be buffered until it's length exceeds this value. `None` means using
    # default value, override this variable to change.
    streaming_amount = None

    def __init__(self, *args, **kwargs):
        super(ProxyView, self).__init__(*args, **kwargs)

        self._rewrite = []
        # Take all elements inside tuple, and insert into _rewrite
        for from_pattern, to_pattern in self.rewrite:
            from_re = re.compile(from_pattern)
            self._rewrite.append((from_re, to_pattern))
        self.http = HTTP_POOLS
        self.log = logging.getLogger('revproxy.view')
        self.log.info("ProxyView created")

    @property
    def upstream(self):
        if not self._upstream:
            raise NotImplementedError('Upstream server must be set')
        return self._upstream

    @upstream.setter
    def upstream(self, value):
        self._upstream = value

    def get_upstream(self, path):
        upstream = self.upstream

        if not getattr(self, '_parsed_url', None):
            self._parsed_url = urlparse(upstream)

        if self._parsed_url.scheme not in ('http', 'https'):
            raise InvalidUpstream(ERRORS_MESSAGES['upstream-no-scheme'] %
                                  upstream)

        if path and upstream[-1] != '/':
            upstream += '/'

        return upstream

    @classonlymethod
    def as_view(cls, **initkwargs):
        view = super(ProxyView, cls).as_view(**initkwargs)
        view.csrf_exempt = True
        return view

    def _format_path_to_redirect(self, request):
        full_path = request.get_full_path()
        self.log.debug("Dispatch full path: %s", full_path)
        for from_re, to_pattern in self._rewrite:
            if from_re.match(full_path):
                redirect_to = from_re.sub(to_pattern, full_path)
                self.log.debug("Redirect to: %s", redirect_to)
                return redirect_to

    def get_proxy_request_headers(self, request):
        """Get normalized headers for the upstream

        Gets all headers from the original request and normalizes them.
        Normalization occurs by removing the prefix ``HTTP_`` and
        replacing and ``_`` by ``-``. Example: ``HTTP_ACCEPT_ENCODING``
        becames ``Accept-Encoding``.

        .. versionadded:: 0.9.1

        :param request:  The original HTTPRequest instance
        :returns:  Normalized headers for the upstream
        """
        return normalize_request_headers(request)

    def get_request_headers(self):
        """Return request headers that will be sent to upstream.

        The header REMOTE_USER is set to the current user
        if AuthenticationMiddleware is enabled and
        the view's add_remote_user property is True.

        .. versionadded:: 0.9.8

        If the view's add_x_forwarded property is True, the
        headers X-Forwarded-For and X-Forwarded-Proto are set to the
        IP address of the requestor and the request's protocol (http or https),
        respectively.

        .. versionadded:: TODO

        """
        request_headers = self.get_proxy_request_headers(self.request)

        if (self.add_remote_user and hasattr(self.request, 'user')
                and self.request.user.is_active):
            request_headers['REMOTE_USER'] = self.request.user.get_username()
            self.log.info("REMOTE_USER set")

        if self.add_x_forwarded:
            request_ip = self.request.META.get('REMOTE_ADDR')
            self.log.debug("Proxy request IP: %s", request_ip)
            request_headers['X-Forwarded-For'] = request_ip

            request_proto = "https" if self.request.is_secure() else "http"
            self.log.debug("Proxy request using %s", request_proto)
            request_headers['X-Forwarded-Proto'] = request_proto

        return request_headers

    def get_quoted_path(self, path):
        """Return quoted path to be used in proxied request"""
        return quote_plus(path.encode('utf8'), QUOTE_SAFE)

    def get_encoded_query_params(self):
        """Return encoded query params to be used in proxied request"""
        get_data = encode_items(self.request.GET.lists())
        return urlencode(get_data)

    def _created_proxy_response(self, request, path):
        request_payload = request.body
        if self.suppress_empty_body and not request_payload:
            request_payload = None

        self.log.debug("Request headers: %s", self.request_headers)

        path = self.get_quoted_path(path)

        request_url = self.get_upstream(path) + path
        self.log.debug("Request URL: %s", request_url)

        if request.GET:
            request_url += '?' + self.get_encoded_query_params()
            self.log.debug("Request URL: %s", request_url)

        try:
            proxy_response = self.http.urlopen(request.method,
                                               request_url,
                                               redirect=False,
                                               retries=self.retries,
                                               headers=self.request_headers,
                                               body=request_payload,
                                               decode_content=False,
                                               preload_content=False)
            self.log.debug("Proxy response header: %s",
                           proxy_response.getheaders())
        except urllib3.exceptions.HTTPError as error:
            self.log.exception(error)
            raise

        return proxy_response

    def _replace_host_on_redirect_location(self, request, proxy_response):
        location = proxy_response.headers.get('Location')
        if location:
            if request.is_secure():
                scheme = 'https://'
            else:
                scheme = 'http://'
            request_host = scheme + request.get_host()

            upstream_host_http = 'http://' + self._parsed_url.netloc
            upstream_host_https = 'https://' + self._parsed_url.netloc

            location = location.replace(upstream_host_http, request_host)
            location = location.replace(upstream_host_https, request_host)
            proxy_response.headers['Location'] = location
            self.log.debug("Proxy response LOCATION: %s",
                           proxy_response.headers['Location'])

    def _set_content_type(self, request, proxy_response):
        content_type = proxy_response.headers.get('Content-Type')
        if not content_type:
            content_type = (mimetypes.guess_type(request.path)[0] or
                            self.default_content_type)
            proxy_response.headers['Content-Type'] = content_type
            self.log.debug("Proxy response CONTENT-TYPE: %s",
                           proxy_response.headers['Content-Type'])

    def dispatch(self, request, path):
        self.request_headers = self.get_request_headers()

        redirect_to = self._format_path_to_redirect(request)
        if redirect_to:
            return redirect(redirect_to)

        proxy_response = self._created_proxy_response(request, path)

        self._replace_host_on_redirect_location(request, proxy_response)
        self._set_content_type(request, proxy_response)

        response = get_django_response(proxy_response,
                                       strict_cookies=self.strict_cookies,
                                       streaming_amount=self.streaming_amount)

        self.log.debug("RESPONSE RETURNED: %s", response)
        return response


class DiazoProxyView(ProxyView, ContextMixin):
    _diazo_rules = None
    diazo_theme_template = 'diazo.html'
    html5 = False

    @property
    def diazo_rules(self):
        if not self._diazo_rules:
            child_class_file = sys.modules[self.__module__].__file__
            app_path = os.path.abspath(os.path.dirname(child_class_file))
            diazo_path = os.path.join(app_path, 'diazo.xml')

            self.log.debug("diazo_rules: %s", diazo_path)
            self._diazo_rules = diazo_path
        return self._diazo_rules

    @diazo_rules.setter
    def diazo_rules(self, value):
        self._diazo_rules = value

    def dispatch(self, request, path):
        response = super(DiazoProxyView, self).dispatch(request, path)

        context_data = self.get_context_data()
        diazo = DiazoTransformer(request, response)
        response = diazo.transform(self.diazo_rules, self.diazo_theme_template,
                                   self.html5, context_data)

        return response




import codecs
import os
import re

from setuptools import setup


def read(*parts):
    return codecs.open(os.path.join(os.path.dirname(__file__), *parts),
                       encoding='utf8').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='arches-django-revproxy',
    description='A temporary fork of django-revproxy to support Arches running Django 3.2',
    version=find_version('revproxy/__init__.py'),
    long_description=read('README.rst'),
    packages=['revproxy'],
    install_requires=[
        'django>=1.7',
        'urllib3>=1.12',
    ],
    extras_require={
        'diazo': ['diazo>=1.0.5', 'lxml>=3.4'],
    },
    tests_require=['mock', 'diazo', 'lxml>=3.4'],
    test_suite="tests.run.runtests",
    author='Sergio Oliveira',
    author_email='sergio@tracy.com.br',
    url='https://github.com/jazzband/django-revproxy',
    license='MPL v2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: Proxy Servers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries',
        ],
)



import logging

from .utils import cookie_from_string, should_stream, set_response_headers

from django.http import HttpResponse, StreamingHttpResponse

logger = logging.getLogger('revproxy.response')


def get_django_response(
    proxy_response, strict_cookies=False, streaming_amount=None
):
    """This method is used to create an appropriate response based on the
    Content-Length of the proxy_response. If the content is bigger than
    MIN_STREAMING_LENGTH, which is found on utils.py,
    than django.http.StreamingHttpResponse will be created,
    else a django.http.HTTPResponse will be created instead

    :param proxy_response: An Instance of urllib3.response.HTTPResponse that
                           will create an appropriate response
    :param strict_cookies: Whether to only accept RFC-compliant cookies
    :param streaming_amount: The amount for streaming HTTP response, if not
                             given, use a dynamic value -- 1("no-buffering")
                             for "text/event-stream" content type, 65535 for
                             other types.
    :returns: Returns an appropriate response based on the proxy_response
              content-length
    """
    status = proxy_response.status
    headers = proxy_response.headers

    logger.debug('Proxy response headers: %s', headers)

    content_type = headers.get('Content-Type')

    logger.debug('Content-Type: %s', content_type)

    if should_stream(proxy_response):
        if streaming_amount is None:
            amt = get_streaming_amt(proxy_response)
        else:
            amt = streaming_amount

        logger.info(('Starting streaming HTTP Response, buffering amount='
                     '"%s bytes"'), amt)
        response = StreamingHttpResponse(proxy_response.stream(amt),
                                         status=status,
                                         content_type=content_type)
    else:
        content = proxy_response.data or b''
        response = HttpResponse(content, status=status,
                                content_type=content_type)

    logger.info('Normalizing response headers')
    set_response_headers(response, headers)

    cookies = proxy_response.headers.getlist('set-cookie')
    logger.info('Checking for invalid cookies')
    for cookie_string in cookies:
        cookie_dict = cookie_from_string(cookie_string,
                                         strict_cookies=strict_cookies)
        # if cookie is invalid cookie_dict will be None
        if cookie_dict:
            response.set_cookie(**cookie_dict)

    logger.debug('Response cookies: %s', response.cookies)

    return response


# Default number of bytes that are going to be read in a file lecture
DEFAULT_AMT = 2**16
# The amount of chunk being used when no buffering is needed: return every byte
# eagerly, which might be bad in performance perspective, but is essential for
# some special content types, e.g. "text/event-stream". Without disabling
# buffering, all events will pending instead of return in realtime.
NO_BUFFERING_AMT = 1

NO_BUFFERING_CONTENT_TYPES = set(['text/event-stream', ])


def get_streaming_amt(proxy_response):
    """Get the value of streaming amount(in bytes) when streaming response

    :param proxy_response: urllib3.response.HTTPResponse object
    """
    content_type = proxy_response.headers.get('Content-Type', '')
    # Disable buffering for "text/event-stream"(or other special types)
    if content_type.lower() in NO_BUFFERING_CONTENT_TYPES:
        return NO_BUFFERING_AMT
    return DEFAULT_AMT
